@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# __coconut_hash__ = 0x96fe352d
+# __coconut_hash__ = 0xc5adeeb4
 
-# Compiled with Coconut version 1.4.3-post_dev46 [Ernest Scribbler]
+# Compiled with Coconut version 1.4.3-post_dev57 [Ernest Scribbler]
 
 """
 The scikit-optimize backend. Does black box optimization using scikit-optimize.
@@ -37,6 +37,87 @@ from bbopt.backends.util import split_examples
 from bbopt.backends.util import make_values
 
 
+# attempt to fix skopt bug by patching sklearn
+try:
+    import sklearn
+    sklearn.utils.fixes.sp_version < (1,)
+except ImportError:
+    pass
+except TypeError:
+    Version = type(sklearn.utils.fixes.sp_version)
+    old_lt = Version.__lt__
+    old_le = Version.__le__
+    old_gt = Version.__gt__
+    old_ge = Version.__ge__
+    try:
+        try:
+            _coconut_dotted_func_name_store_0 = __lt__
+        except _coconut.NameError:
+            _coconut_dotted_func_name_store_0 = _coconut_sentinel
+        def __lt__(self, other):
+            try:
+                result = old_lt(self, other)
+            except (TypeError, NotImplementedError):
+                result = NotImplemented
+            if result is NotImplemented:
+                return self.release < other
+            else:
+                return result
+        Version.__lt__ = __lt__
+        if _coconut_dotted_func_name_store_0 is not _coconut_sentinel:
+            __lt__ = _coconut_dotted_func_name_store_0
+        try:
+            _coconut_dotted_func_name_store_1 = __le__
+        except _coconut.NameError:
+            _coconut_dotted_func_name_store_1 = _coconut_sentinel
+        def __le__(self, other):
+            try:
+                result = old_le(self, other)
+            except (TypeError, NotImplementedError):
+                result = NotImplemented
+            if result is NotImplemented:
+                return self.release <= other
+            else:
+                return result
+        Version.__le__ = __le__
+        if _coconut_dotted_func_name_store_1 is not _coconut_sentinel:
+            __le__ = _coconut_dotted_func_name_store_1
+        try:
+            _coconut_dotted_func_name_store_2 = __gt__
+        except _coconut.NameError:
+            _coconut_dotted_func_name_store_2 = _coconut_sentinel
+        def __gt__(self, other):
+            try:
+                result = old_gt(self, other)
+            except (TypeError, NotImplementedError):
+                result = NotImplemented
+            if result is NotImplemented:
+                return self.release > other
+            else:
+                return result
+        Version.__gt__ = __gt__
+        if _coconut_dotted_func_name_store_2 is not _coconut_sentinel:
+            __gt__ = _coconut_dotted_func_name_store_2
+        try:
+            _coconut_dotted_func_name_store_3 = __ge__
+        except _coconut.NameError:
+            _coconut_dotted_func_name_store_3 = _coconut_sentinel
+        def __ge__(self, other):
+            try:
+                result = old_ge(self, other)
+            except (TypeError, NotImplementedError):
+                result = NotImplemented
+            if result is NotImplemented:
+                return self.release >= other
+            else:
+                return result
+        Version.__ge__ = __ge__
+        if _coconut_dotted_func_name_store_3 is not _coconut_sentinel:
+            __ge__ = _coconut_dotted_func_name_store_3
+    except TypeError:
+        pass
+
+
 # Utilities:
 
 def create_space(name, func, *args):
