@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# __coconut_hash__ = 0x26e2b217
+# __coconut_hash__ = 0x564ce5ce
 
-# Compiled with Coconut version 1.4.3-post_dev46 [Ernest Scribbler]
+# Compiled with Coconut version 1.4.3-post_dev57 [Ernest Scribbler]
 
 """
 BBopt command line interface.
@@ -33,19 +33,22 @@ import subprocess
 from concurrent.futures import ProcessPoolExecutor
 from pprint import pprint
 
+if sys.version_info >= (3, 3):
+    from concurrent.futures.process import BrokenProcessPool
+else:
+    BrokenProcessPool = KeyboardInterrupt
+
+from bbopt import constants
 from bbopt.optimizer import BlackBoxOptimizer
-from bbopt.constants import description
-from bbopt.constants import default_trials
-from bbopt.constants import default_jobs
 
 
-parser = argparse.ArgumentParser(prog="bbopt", description=description)
+parser = argparse.ArgumentParser(prog="bbopt", description=constants.description)
 
 parser.add_argument("file", metavar="file", type=str, help="path to the Python file to run")
 
-parser.add_argument("-n", "--num-trials", metavar="trials", type=int, default=default_trials, help="number of trials to run (defaults to {_coconut_format_0})".format(_coconut_format_0=(default_trials)))
+parser.add_argument("-n", "--num-trials", metavar="trials", type=int, default=constants.default_trials, help="number of trials to run (defaults to {_coconut_format_0})".format(_coconut_format_0=(constants.default_trials)))
 
-parser.add_argument("-j", "--jobs", metavar="processes", type=int, default=default_jobs, help="number of processes to use (defaults to {_coconut_format_0})".format(_coconut_format_0=(default_jobs)))
+parser.add_argument("-j", "--jobs", metavar="processes", type=int, default=constants.default_jobs, help="number of processes to use (defaults to {_coconut_format_0})".format(_coconut_format_0=(constants.default_jobs)))
 
 parser.add_argument("-q", "--quiet", action="store_true", help="suppress all informational output")
 
@@ -62,14 +65,17 @@ def base_show(quiet, msg):
 
 def run_trial(args, cmd, i):
     """Pickleable function for running trials in parallel."""
-    show = _coconut.functools.partial(base_show, args.quiet)
-    show("{_coconut_format_0}/{_coconut_format_1} starting...".format(_coconut_format_0=(i + 1), _coconut_format_1=(args.num_trials)))
-    subprocess.check_call(cmd)
-    show("{_coconut_format_0}/{_coconut_format_1} finished.".format(_coconut_format_0=(i + 1), _coconut_format_1=(args.num_trials)))
+    try:
+        show = _coconut.functools.partial(base_show, args.quiet)
+        show("{_coconut_format_0}/{_coconut_format_1} starting...".format(_coconut_format_0=(i + 1), _coconut_format_1=(args.num_trials)))
+        subprocess.check_call(cmd)
+        show("{_coconut_format_0}/{_coconut_format_1} finished.".format(_coconut_format_0=(i + 1), _coconut_format_1=(args.num_trials)))
+    except BrokenProcessPool:
+        raise KeyboardInterrupt()
 
 
-def main():
-    args = parser.parse_args()
+def main(*args, **kwargs):
+    args = parser.parse_args(*args, **kwargs)
     if not os.path.isfile(args.file):
         raise ValueError("could not find file {_coconut_format_0}".format(_coconut_format_0=(args.file)))
 
