@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# __coconut_hash__ = 0xf7afb2cd
+# __coconut_hash__ = 0xce5d68a5
 
-# Compiled with Coconut version 1.4.3-post_dev46 [Ernest Scribbler]
+# Compiled with Coconut version 1.4.3-post_dev57 [Ernest Scribbler]
 
 """
 The main BBopt interface.
@@ -35,9 +35,11 @@ else:
 import math
 import itertools
 import time
+from pprint import pprint
 
 import numpy as np
 
+from bbopt import constants
 from bbopt.registry import backend_registry
 from bbopt.registry import init_backend
 from bbopt.registry import alg_registry
@@ -54,11 +56,8 @@ from bbopt.util import sorted_examples
 from bbopt.util import running_best
 from bbopt.util import plot
 from bbopt.util import open_with_lock
-from bbopt.constants import data_file_ext
-from bbopt.constants import default_alg
-from bbopt.constants import default_protocol
+from bbopt.util import printerr
 from bbopt.backends.serving import ServingBackend
-from bbopt.backends.skopt import SkoptBackend
 
 
 def array_param(func, name, shape, kwargs):
@@ -74,9 +73,14 @@ def array_param(func, name, shape, kwargs):
     return arr
 
 
+DEFAULT_ALG_SENTINEL = object()
+
+
 class BlackBoxOptimizer(_coconut.object):
     """Main bbopt optimizer object. See https://github.com/evhub/bbopt for documentation."""
     backend = None
+    _new_params = None
+    _current_example = None
 
     @_coconut_mark_as_match
     def __init__(*_coconut_match_to_args, **_coconut_match_to_kwargs):
@@ -106,7 +110,7 @@ class BlackBoxOptimizer(_coconut.object):
 # auto-detect protocol
             self._protocol = "json"
             if not os.path.exists(self.data_file):
-                self._protocol = default_protocol
+                self._protocol = constants.default_protocol
         else:
             self._protocol = protocol
 
@@ -154,6 +158,9 @@ class BlackBoxOptimizer(_coconut.object):
         """Load the given examples into memory."""
         for ex in examples:
             if ex not in self._examples:
+                for name, val in (list)(ex["values"].items()):
+                    func, args, kwargs = (lambda _coconut_x: self._old_params[name] if _coconut_x is None else _coconut_x)((lambda _coconut_x: None if _coconut_x is None else _coconut_x.get(name))(self._new_params))
+                    ex["values"][name] = param_processor.verify_support(name, val, func, *args, **kwargs)
                 self._examples.append(ex)
 
     def _load_from(self, df):
@@ -208,6 +215,7 @@ class BlackBoxOptimizer(_coconut.object):
 
     def _get_skopt_backend(self):
         """Get a scikit-optimize backend regardless of whether currently using one."""
+        from bbopt.backends.skopt import SkoptBackend
         if isinstance(self.backend, SkoptBackend):
             return self.backend
 
@@ -244,7 +252,7 @@ class BlackBoxOptimizer(_coconut.object):
                 _coconut_match_check = True
         if _coconut_match_check:
             if (func, args) != (old_func, old_args):
-                print("BBopt Warning: detected change in parameter {_coconut_format_0} ({_coconut_format_1} != {_coconut_format_2}) (you may need to delete your old BBopt data)".format(_coconut_format_0=(name), _coconut_format_1=((func, args)), _coconut_format_2=((old_func, old_args))))
+                printerr("BBopt Warning: detected change in parameter {_coconut_format_0} ({_coconut_format_1} != {_coconut_format_2}) (you may need to delete your old BBopt data)".format(_coconut_format_0=(name), _coconut_format_1=((func, args)), _coconut_format_2=((old_func, old_args))))
 
         value = self.backend.param(name, func, *args, **kwargs)
         self._new_params[name] = (func, args, kwargs)
@@ -269,9 +277,11 @@ class BlackBoxOptimizer(_coconut.object):
         """All algorithms supported by run."""
         return alg_registry.asdict()
 
-    def run(self, alg=default_alg):
+    def run(self, alg=DEFAULT_ALG_SENTINEL):
         """Optimize parameters using the given algorithm
         (use .algs to get the list of valid algorithms)."""
+        if alg is DEFAULT_ALG_SENTINEL:
+            alg = constants.default_alg
         backend, options = alg_registry[alg]
         self.run_backend(backend, **options)
 
@@ -302,12 +312,15 @@ class BlackBoxOptimizer(_coconut.object):
     @property
     def data_file(self):
         """The path to the file we are saving data to."""
-        return os.path.join(os.path.dirname(self._file), self._file_name) + data_file_ext + (".json" if self._using_json else ".pickle")
+        return os.path.join(os.path.dirname(self._file), self._file_name) + constants.data_file_ext + (".json" if self._using_json else ".pickle")
 
-    def get_data(self):
+    def get_data(self, print_data=False):
         """Get all currently-loaded data as a dictionary containing params and examples."""
         self._old_params.update(self._new_params)
-        return {"params": self._old_params, "examples": self._examples}
+        data_dict = {"params": self._old_params, "examples": self._examples}
+        if print_data:
+            pprint(data_dict)
+        return data_dict
 
     @property
     def num_examples(self):
@@ -326,10 +339,9 @@ class BlackBoxOptimizer(_coconut.object):
 
     def get_current_run(self):
         """Return a dictionary containing the current parameters and reward."""
-        try:
-            return self._current_example
-        except AttributeError:
+        if self._current_example is None:
             raise ValueError("get_current_run calls must come after run")
+        return self._current_example
 
     def get_optimal_run(self):
         """Return a dictionary containing the optimal parameters and reward computed so far."""
@@ -420,10 +432,6 @@ class BlackBoxOptimizer(_coconut.object):
         """Create a new parameter with the given name modeled by random.randrange(*args)."""
         return self.param(name, "randrange", *args, **kwargs)
 
-    def choice(self, name, seq, **kwargs):
-        """Create a new parameter with the given name modeled by random.choice(seq)."""
-        return self.param(name, "choice", seq, **kwargs)
-
     def uniform(self, name, a, b, **kwargs):
         """Create a new parameter with the given name modeled by random.uniform(a, b)."""
         return self.param(name, "uniform", a, b, **kwargs)
@@ -460,6 +468,22 @@ class BlackBoxOptimizer(_coconut.object):
         """Create a new parameter with the given name modeled by random.weibullvariate(alpha, beta)."""
         return self.param(name, "weibullvariate", alpha, beta, **kwargs)
 
+# Choice functions:
+
+    def _categorical(self, name, num_categories, **kwargs):
+        """Create a new parameter with the given name modeled by random.choice(range(num_categories))."""
+        if constants.use_randrange_for_categorical_data:
+            return self.randrange(name, num_categories, **kwargs)
+        else:
+            return self.param(name, "choice", range(num_categories), **kwargs)
+
+    def choice(self, name, seq, **kwargs):
+        """Create a new parameter with the given name modeled by random.choice(seq)."""
+        if constants.use_generic_categories_for_categorical_data:
+            return seq[self._categorical(name, len(seq), **kwargs)]
+        else:
+            return self.param(name, "choice", seq, **kwargs)
+
 # Derived random functions:
 
     def randint(self, name, a, b, **kwargs):
@@ -492,7 +516,7 @@ class BlackBoxOptimizer(_coconut.object):
 
     def randbool(self, name, **kwargs):
         """Create a new boolean parameter with the given name."""
-        return self.choice(name, [False, True], **kwargs)
+        return bool(self.choice(name, [False, True], **kwargs))
 
     def sample(self, name, population, k, **kwargs):
         """Create a new parameter with the given name modeled by random.sample(population, k)."""
@@ -508,7 +532,7 @@ class BlackBoxOptimizer(_coconut.object):
                     elem = _coconut_igetitem(val, i)
                     return sampling_population.index(elem) if elem in sampling_population else 0
                 proc_kwargs = (param_processor.modify_kwargs)(_coconut_lambda_0, kwargs)
-                ind = self.randrange("{_coconut_format_0}[{_coconut_format_1}]".format(_coconut_format_0=(name), _coconut_format_1=(i)), len(sampling_population), **proc_kwargs)
+                ind = self._categorical("{_coconut_format_0}[{_coconut_format_1}]".format(_coconut_format_0=(name), _coconut_format_1=(i)), len(sampling_population), **proc_kwargs)
                 sample.append(sampling_population.pop(ind))
         return sample
 
